#! /usr/bin/env python
import sys
from optparse import OptionParser
import os
import shutil
import urllib
from glob import glob

from util import run_command, fatal, CommandError, which
import constants
from xml.dom import minidom as dom

def get_ns3_info(ns3_branch):
    '''Returns a tuple containing the NS-3 directory and Mercurial repository URL.'''

    ns3_dir = os.path.split(ns3_branch)[-1]
    ns3_branch_url = constants.NSNAM_CODE_BASE_URL + ns3_branch

    return (ns3_dir, ns3_branch_url)


def get_pybindgen_info(ns3_dir):
    '''Returns a tuple containing the Pybindgen directory and the Bazaar repository version number.'''

    if sys.platform in ['cygwin']:
        print "Architecture (%s) does not support PyBindGen ... skipping" % (sys.platform,)
        raise RuntimeError

    # (peek into the ns-3 wscript and extract the required pybindgen version)
    ns3_python_wscript = open(os.path.join(ns3_dir, "bindings", "python", "wscript"), "rt")
    required_pybindgen_version = None
    for line in ns3_python_wscript:
        if 'REQUIRED_PYBINDGEN_VERSION' in line:
            required_pybindgen_version = eval(line.split('=')[1].strip())
            ns3_python_wscript.close()
            break
    if required_pybindgen_version is None:
        fatal("Unable to detect pybindgen required version")
    print "Required pybindgen version: ", '.'.join([str(x) for x in required_pybindgen_version])

    return (constants.LOCAL_PYBINDGEN_PATH, '.'.join([str(x) for x in required_pybindgen_version]), required_pybindgen_version)


def get_nsc_info(ns3_dir):
    '''Returns a tuple containing the NSC directory and NSC version number.'''

    # Skip downloading NSC on OS X due to HFS+ case insensitive issues
    # Skip downloading NSC on Cygwin because of fundamental incompatibilities.
    if sys.platform in ['darwin', 'cygwin']:
        print "Architecture (%s) does not support NSC... skipping" % (sys.platform,)
        raise RuntimeError

    # (peek into the ns-3 wscript and extract the required nsc version)
    internet_stack_wscript = open(os.path.join(ns3_dir, "src", "internet-stack", "wscript"), "rt")
    required_nsc_version = None
    for line in internet_stack_wscript:
        if 'NSC_RELEASE_NAME' in line:
            required_nsc_version = eval(line.split('=')[1].strip())
            break
    internet_stack_wscript.close()
    if required_nsc_version is None:
        fatal("Unable to detect NSC required version")
    print "Required NSC version: ", required_nsc_version
    
    return (constants.LOCAL_NSC_PATH, required_nsc_version)


def main():
    if not which("jhbuild"):
      fatal("failed:  jhbuild required but not found on system")
      
    parser = OptionParser()
    parser.add_option("-n", "--ns3-branch", dest="ns3_branch", default="ns-3-dev",
                      help="Name of the ns-3 repository", metavar="BRANCH_NAME")
    (options, dummy_args) = parser.parse_args()

    # first of all, change to the directory of the script
    os.chdir(os.path.abspath(os.path.dirname(__file__)))

    # Create the configuration file
    config = dom.getDOMImplementation().createDocument(None, "config", None)

    # Generate the JHBuild Python configuration file.
    jhbuildrc_name = "download.jhbuildrc"
    jhbuildrc_file = open(jhbuildrc_name, "w")
    jhbuildrc_file.write("#! /usr/bin/env python\n")
    jhbuildrc_file.write("\n")
    jhbuildrc_file.write("# Module set to use.\n")
    jhbuildrc_file.write("moduleset = 'ns-3.modules'\n")
    jhbuildrc_file.write("\n")
    jhbuildrc_file.write("# A list of the modules to build.\n")
    jhbuildrc_file.write("modules = ['ns-3-dev', 'pybindgen', 'nsc']\n")
    jhbuildrc_file.write("\n")
    jhbuildrc_file.write("# The directory source should be checked out to.\n")
    jhbuildrc_file.write("checkoutroot = '.'\n")
    jhbuildrc_file.write("\n")
    jhbuildrc_file.write("# The prefix to configure/install modules to (must have write access.)\n")
    jhbuildrc_file.write("prefix = 'install'\n")
    jhbuildrc_file.close()

    # work around http_proxy handling bug in bzr
    if 'http_proxy' in os.environ and 'https_proxy' not in os.environ:
        os.environ['https_proxy'] = os.environ['http_proxy']
 
    # Run JHBuild to download the essential NS-3 files for the modules
    # without building anything.
    run_command(['jhbuild', '--file', jhbuildrc_name, 'update'])

    # Post-download custom steps

    # Get configuration information for the modules.
    (ns3_dir, ns3_branch_url) = get_ns3_info(options.ns3_branch)
    print "NS-3 directory: ", ns3_dir
    print "NS-3 repository: ", ns3_branch_url
    (pybindgen_dir, pybindgen_version, required_pybindgen_version) = get_pybindgen_info(ns3_dir)
    (nsc_dir, nsc_version) = get_nsc_info(ns3_dir)

    # Set configuration information for NS-3.
    ns3_config = config.documentElement.appendChild(config.createElement("ns-3"))
    ns3_config.setAttribute("dir", ns3_dir)
    ns3_config.setAttribute("branch", options.ns3_branch)

    ## generate a fake version.py file in pybindgen it's safer this
    ## way, since the normal version generation process requires
    ## bazaar python bindings, which may not be available.
    vfile = open(os.path.join(constants.LOCAL_PYBINDGEN_PATH, "pybindgen", "version.py"), "wt")
    vfile.write("""
# (fake version generated by ns-3)
__version__ = %r
""" % list(required_pybindgen_version))
    vfile.close()

    # Set configuration information for pybindgen.
    pybindgen_config = config.documentElement.appendChild(config.createElement("pybindgen"))
    pybindgen_config.setAttribute("dir", pybindgen_dir)
    pybindgen_config.setAttribute("version", pybindgen_version)

    # Set configuration information for NSC.
    nsc_config = config.documentElement.appendChild(config.createElement("nsc"))
    nsc_config.setAttribute("dir", nsc_dir)
    nsc_config.setAttribute("version", nsc_version)

    # write the config to a file
    dot_config = open(".config", "wt")
    config.writexml(dot_config, addindent="    ", newl="\n")
    dot_config.close()

    return 0

if __name__ == '__main__':
    sys.exit(main())
